# -*- coding: utf-8 -*-

from twisted.web.client import getPage
from twisted.internet import defer
from simplejson import loads as json_loads, dumps as json_dumps
import logging
import base64
import datetime

log = logging.getLogger("nozbe")

PRODUCTION_URL = "http://www.nozbe.com"
DEVEL_URL = "http://devl.nozbe.net"

class NozbeBase(object):
    """
    Wspólna część składowa (i konstruktor) dwóch poniższych klas.
    """
    def __init__(self, api_key, devel = None):
        """
        Inicjalizacja. api_key to klucz dostępu, devel - jeśli niepuste
        - znaczy używanie środowiska developerskiego i powinno być
        słownikiem user/password z danymi do autentyfikacji HTTP.
        """
        self.key = api_key
        if not devel:
            self.url = PRODUCTION_URL
            self.headers = {}
        else:
            self.url = DEVEL_URL
            assert(type(devel) == dict)
            basic_auth = base64.encodestring('%(user)s:%(password)s' % devel)
            auth_header = "Basic " + basic_auth.strip()
            self.headers = dict(Authorization = auth_header)

    @defer.inlineCallbacks
    def _make_get_request(self, api_url, *call_items, **params):
        """
        Generuje żądanie do Nozbe.

        @param call_items: projects, context albo i actions, what-context
        @param args: ewentualne specyficzne parametry, np id=cos
        """
        url = self._make_url(api_url, *call_items, **params)
        log.debug("Getting " + url)
        reply = yield getPage(url, headers = self.headers)
        if not reply:
            raise Exception("Can not load data from Nozbe. Wrong API key?")
        log.debug("Reply: " + reply)
        data = json_loads(reply)
        defer.returnValue(data)

    @defer.inlineCallbacks
    def _make_post_request(self, api_url, *call_items, **params):
        """
        Generuje POSTa. Działa identycznie jak _make_get_request, ale wśród params
        musi być POST=(słownik)
        """
        post_text = json_dumps(params['POST'])
        del params['POST']
        url = self._make_url(api_url, *call_items, **params)
        log.debug("Posting " + url + " with " + post_text)
        reply = yield getPage(url, headers = self.headers, method = "POST", postdata = post_text)
        if not reply:
            raise Exception("Can not save data to Nozbe. Wrong API key?")
        log.debug("Reply: " + reply)
        data = json_loads(reply)
        defer.returnValue(data)

    def _make_url(self, api_url, *call_items, **params):
        parts = [self.url, api_url] + list(call_items) \
            + [ ("%s-%s" % (key, value)) for key, value in params.iteritems() ] \
            + [ "key-%s" % self.key ]
        url = "/".join(parts)
        if type(url) == unicode:
            url = url.encode("utf-8")
        return url

class NozbeLoader(NozbeBase):
    """
    @obsolete

    Obsługa pobierania danych z Nozbe przy pomocy *starego* API
    """
    api_url = "api"

    def get_projects(self):
        return self._make_request("projects")
    def get_contexts(self):
        return self._make_request("contexts")
    def get_project_actions(self, project_id):
        return self._make_request("actions", "what-project", id = project_id)

    def _make_request(self, *call_items, **params):
        return self._make_get_request(self.api_url, *call_items, **params)

class NozbeSynchr(NozbeBase):
    """
    Obsługa pobierania danych z Nozbe oraz ich zapisu przy pomocy sync-api.

    Funkcje get_* pobierają rozmaite dane, zwracając odpowiednie słowniki i listy słowników.

    Funkcje add_*, delete_* i update_* służą do modyfikacji, ale same niczego nie zmieniają!
    Lista zmian zapisana przy pomocy sekwencji add_, delete_, update_ zostaje faktycznie
    wykonana przez wywołanie save_changes.
    """

    api_url = "sync"
    app_id = "997"

    RECUR_NO_REPEAT = 0
    RECUR_EVERY_DAY = 1
    RECUR_EVERY_WEEKDAY = 2
    RECUR_EVERY_WEEK = 3
    RECUR_EVERY_TWO_WEEKS = 4
    RECUR_EVERY_MONTH = 5
    RECUR_EVERY_HALF_A_YEAR = 6
    RECUR_EVERY_YEAR = 7

    def __init__(self, api_key, devel = None):
        NozbeBase.__init__(self, api_key, devel)
        self._reset_pending_updates()
        self._update_id = 1

    def get_projects(self):
        """
        Zwraca listę projektów. Każdy projekt to słownik o polach
        - hash,
        - name,
        - body,
        - tag (rozdzielone spacjami),
        - count (ile akcji nieskończonych),
        - share (lista współwłaścicieli - name, hash),
        - ts
        """
        return self._get_data("project")
    def get_contexts(self):
        """
        Zwraca listę kontekstów. Słowniki o polach:
        - hash,
        - name,
        - body,
        - icon,
        - count (ile niesk. akcji)
        - ts
        """
        return self._get_data("context")
    def get_tasks(self):
        """
        Zwraca listę akcji (zadań). Słowniki o polach:
        - hash,
        - name,
        - name_show, (textile applied)
        - date     (data zakończenia)
        - datetime (due date)
        - recur    (patrz stałe RECUR_)
        - time     (koszt - czas w minutach)
        - context_id
        - context_hash
        - project_id
        - project_hash
        - next     (1 - "next action", 0 - nie)
        - re_user  (0 gdy odpowiadam ja, inaczej hash innego usera na którym wisi)
        - ts
        """
        return self._get_data("task")
    def get_notes(self):
        """
        Zwraca listę notek. Słowniki:
        - hash
        - name
        - body
        - body_show
        - date
        - context_id
        - context_hash
        - project_id
        - project_hash
        - ts
        """
        return self._get_data("note")
    def get_uploads(self):
        """
        Zwraca listę uploadów. Słowniki:
        - hash,
        - name,
        - date,
        - size (bajty)
        - context_id
        - context_hash
        - project_id
        - project_hash
        - ts
        """
        return self._get_data("upload")
    def get_all(self):
        """
        Zwraca słownik o kluczach project, context, task, note, upload, pod każdym kluczem lista
        taka jak opisana powyżej
        """
        return self._make_get_request(self.api_url, "getdata", dev=self.app_id, what="all")
    def get_ts(self):
        """
        Zwraca serwerowy timestamp dla wszystkich rodzajów danych (tj. info kiedy
        ostatnio były zmieniane). Wynikiem jest hasz o odpowiednich kluczach.
        """
        return self._make_get_request(self.api_url, "getts")

    ### Funkcje modyfikujące

    def add_project(self, name, body = "", tag = ""):
        self._add_record('project',
                         dict(name = name, body = body, tag = tag))

    def update_project(self, record):
        self._update_record('project', record.copy())

    def delete_project(self, record):
        self._delete_record('project', record.copy())

    def add_context(self, name, body = "", icon = ""):
        self._add_record('context',
                         dict(name = name, body = body, icon = icon))

    def update_context(self, record):
        self._update_record('context', record.copy())

    def delete_context(self, record):
        self._delete_record('context', record.copy())

    def add_tag(self, name):
        self._add_record('tag',
                         dict(name = name))

    def update_tag(self, record):
        self._update_record('tag', record.copy())

    def delete_tag(self, record):
        self._delete_record('tag', record.copy())

    def add_task(self, name, project_hash, context_hash = "",
                 date = "", datetime = "", recur = 0, # NozbeSynchr.RECUR_NO_REPEAT,
                 time = "5", next = 0):
        self._add_record('task',
                         dict(name = name, project_hash = project_hash, context_hash = context_hash,
                              date = date, datetime = datetime, recur = recur, time = time, next = next))
    def update_task(self, record):
        self._update_record('task', record.copy())

    def delete_task(self, record):
        self._delete_record('task', record.copy())

    def add_note(self, name, body, project_hash, context_hash = "", date = ""):
        self._add_record('note',
                         dict(name = name, body = body,
                              project_hash = project_hash, context_hash = context_hash, date = date))

    def update_note(self, record):
        self._update_record('note', record.copy())

    def delete_note(self, record):
        self._delete_record('note', record.copy())

    def _add_record(self, what, record):
        self._update_id += 1
        record['id'] = self._update_id
        self._pending_updates[what].append(record)

    def _update_record(self, what, record):
        record['flag'] = 'modified'
        record['ts'] =  datetime.datetime.now().strftime("%Y-%m-%d %T")
        self._pending_updates[what].append(record)

    def _delete_record(self, what, record):
        record['flag'] = 'deleted'
        record['ts'] =  datetime.datetime.now().strftime("%Y-%m-%d %T")
        self._pending_updates[what].append(record)

    def save_changes(self):
        """
        Zapisuje wszystkie zmiany przygotowane przy pomocy funkcji add_*, update_*, delete_*
        """
        updates = self._pending_updates
        self._reset_pending_updates()
        for key in updates.keys():
            if not updates[key]:
                del updates[key]
        return self._make_post_request(self.api_url, "process", dev = self.app_id,
                                       POST = updates)

    def _reset_pending_updates(self):
        self._pending_updates = dict(project = [], task = [], context = [], tag = [], note = [])

    @defer.inlineCallbacks
    def _get_data(self, what):
        r = yield self._make_get_request(self.api_url, "getdata", dev=self.app_id, what=what)
        defer.returnValue( r[what] )