# -*- coding: utf-8 -*-

from mekk.xmind import XMindDocument, XMIND_MARKS
from nozbe import NozbeLoader
import simplejson
from twisted.internet import defer
import context_icons

# Znaczniki używane do kontekstów
NEXT_ACTION_MARK = 'task-start'
DONE_MARK = 'task-done'

# URLe
PRODUCTION_PROJECT_URL = "http://www.nozbe.com/account/projects/show-%s"
DEVEL_PROJECT_URL = "http://devl.nozbe.net/account/projects/show-%s"

class MarkerGen(object):
    """
    Klasa przypisująca ikonom kontekstów z Nozbe markery dla XMinda
    """
    def __init__(self):
        self.ci = context_icons.IconsMapper()
    def give_mark(self, icon):
        if icon is None or icon == "icon-.png":
            return None
        return self.ci.id_for_icon(icon)

@defer.inlineCallbacks
def nozbe_to_xmind(loader, xmind_filename, devel = False):
    """
    Podłącza się do zadanego konta XMind, pobiera projekty,
    konteksty i akcje i wszystko zapisuje

    @param loader: obiekt ładujący (zwykle NozbeLoader(api_key) ale można do testów inne)
    @param xmind_filename: tworzony plik .xmind
    @param devel: ustawiane gdy używamy sajtu developerskiego
    """
    all_actions = []

    if not devel:
        project_url = PRODUCTION_PROJECT_URL
    else:
        project_url = DEVEL_PROJECT_URL

    xmw = XMindDocument.create("Projects", "Projects")
    sheet = xmw.get_first_sheet()
    project_root = sheet.get_root_topic()
    project_root.set_note("View the Help sheet for info\nwhat you can do with this map")

    marker_gen = MarkerGen()

    print "Loading projects..."
    projects = yield loader.get_projects()
    if projects is None:
        projects = []
    print "   DONE."

    project_topics = dict()       # id projektu -> Topic

    project_style = xmw.create_topic_style(fill = "#37D02B")

    for p in projects:
        p_id = p['id']
        p_topic = project_root.add_subtopic(p['name'], p_id)
        p_topic.set_link(project_url % p_id)
        p_topic.set_style(project_style)

        print "Loading actions from project", p['name']
        actions = yield loader.get_project_actions(p_id)
        print "   DONE."
        if actions is None:
            actions = []

        for action in actions:
            # Każda akcja ma 'id', 'name', 'name_show', 'done', 'time', 'project_id',
            # 'project_name', 'context_id', 'context_name', 'context_icon', 'next'
            a_topic = p_topic.add_subtopic(action['name'], action['id'])
            if action['next']:
                a_topic.add_marker(NEXT_ACTION_MARK)
            if action['done']:
                a_topic.add_marker(DONE_MARK)
            c_mark = marker_gen.give_mark(action['context_icon'])
            if c_mark:
                a_topic.add_marker(c_mark)

        all_actions.extend(actions)

    # Legenda
    legend = sheet.get_legend()
    legend.add_marker(NEXT_ACTION_MARK, "Next action")
    legend.add_marker(DONE_MARK, "Done")

    print "Loading contexts"
    contexts = yield loader.get_contexts()
    print "   DONE."
    if contexts:
        #context_root = legend_root.add_subtopic("Contexts")
        for c in contexts:
            #c_id = c["id"]
            c_mark = marker_gen.give_mark(c["icon"])
            if c_mark:
                legend.add_marker(c_mark, c["name"])
            #c_topic = context_root.add_subtopic(c["name"], c_id)
            #c_topic.set_link("http://www.nozbe.com/account/contexts/show-" + c_id)

    #data_root = project_root.add_subtopic(
    #    "Do not remove",
    #    detached = True)

    info_sheet = xmw.create_sheet("Help", "What you can do\nwith this map")
    info_root = info_sheet.get_root_topic()
    info_root.set_note("""Do not remove the attachment! It is necessary to save your changes back.""")
    info_root.set_attachment(
        simplejson.dumps(dict(projects = projects, contexts = contexts, actions = all_actions)),
        ".json")
    e = info_root.add_subtopic("Everything")
    e.add_subtopic("But only the following changes\nwill be saved back to Nozbe")
    e.add_subtopic("All the rest\nwill be ignored\n").add_subtopic("Including deletes")
    p = info_root.add_subtopic("Move actions\nbetween projects")
    p.add_subtopic("But don't make an action\na child of an action")
    info_root.add_subtopic("Add new actions").add_subtopic("Under projects")
    r = info_root.add_subtopic("Rename")
    r.add_subtopic("Projects")
    r.add_subtopic("Actions")
    c = info_root.add_subtopic("Flag")
    c.add_subtopic("Next actions")
    c.add_subtopic("Contexts")
    info_root.add_subtopic("Save changes back to Nozbe").add_subtopic(
            "Save edited map and run\nthe script to upload your changes\nto Nozbe")

    xmw.save(xmind_filename)

